@@ -1,26 +1,6 @@
-# Copyright (c) 2006 by BlueDynamics Tyrol - Klein und Partner KEG, Austria
-#
+# Copyright (c) 2006-2009 BlueDynamics Alliance, Austria http://bluedynamics.com
 # GNU General Public License (GPL)
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-# 02110-1301, USA.
-#
 
-__author__ = """Jens Klein <jens@bluedynamics.com>,
-                Robert Niederreiter <robertn@bluedynamics.com>"""
-__docformat__ = 'plaintext'
 import logging
 logger = logging.getLogger('PASGroupsFromLDAP')
 
@@ -38,12 +18,13 @@ from Products.PluggableAuthService.utils import createViewName
 from Products.PluggableAuthService.utils import classImplements
 from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
 from Products.PluggableAuthService.permissions import ManageGroups
-
 from Products.PluggableAuthService.UserPropertySheet import UserPropertySheet
-from Products.PluggableAuthService.interfaces.plugins import IGroupsPlugin
-from Products.PluggableAuthService.interfaces.plugins import IGroupEnumerationPlugin
-from Products.PluggableAuthService.interfaces.plugins import IRolesPlugin
-from Products.PluggableAuthService.interfaces.plugins import IPropertiesPlugin
+from Products.PluggableAuthService.interfaces.plugins import (
+    IGroupsPlugin,
+    IGroupEnumerationPlugin,
+    IRolesPlugin,
+    IPropertiesPlugin,
+)
 from Products.PlonePAS.interfaces.group import IGroupIntrospection
 from Products.PlonePAS.plugins.group import PloneGroup
 from zope.interface import Interface, implements
@@ -65,7 +46,9 @@ def manage_addGroupsFromLDAPMultiPlugin(self, id, title, server, port,
                                         managerdn, password, groupdn, scope,
                                         objectclasses, escapevalues,
                                         attributeid, attributetitle,
-                                        attributemembers, REQUEST=None):
+                                        attributemembers, 
+                                        attributegroupattr=None, 
+                                        REQUEST=None):
     """Factory method to instantiate a GroupsFromLDAPMultiPlugin.
     """
     # Make sure we really are working in our container (the
@@ -90,6 +73,7 @@ def manage_addGroupsFromLDAPMultiPlugin(self, id, title, server, port,
                 'attributeid': attributeid,
                 'attributetitle': attributetitle,
                 'attributemembers': attributemembers,
+                'attributegroupattr': attributegroupattr,
             },
         }
     else:
@@ -113,6 +97,11 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
 
     meta_type = 'GroupsFromLDAPMultiPlugin'
 
+    config_whitelist=['server','port','managerdn','password',
+                      'groupdn','scope','objectclasses','escapevalues',
+                      'attributeid','attributetitle','attributemembers',
+                      'attributegroupattr',]
+
     def __init__(self, id, title=None):
         self.id = id
         self.title = title
@@ -133,6 +122,7 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
         self.config['attributeid'] = 'cn'
         self.config['attributemembers'] = 'memberUid'
         self.config['attributetitle'] = 'cn'
+        self.config['attributegroupattr'] = ''
         self.initializeLDAP()
 
     #---------------------------------------------------------------------------
@@ -148,15 +138,29 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
         o May assign groups based on values in the REQUEST object, if present
           (optional, you also can ignore it)
         """
-
         userid = principal.getUserId()
+        groupattr = self.config.get('attributegroupattr',None)
+        if groupattr:
+            plugins = self.acl_users._getOb('plugins')
+            propfinders = plugins.listPlugins(IPropertiesPlugin)
+            id_for_group = None 
+            for propfinder in propfinders:
+                ps = propfinder[1].getPropertiesForUser(principal)
+                if ps:
+                    if hasattr(ps,'get'):
+                        id_for_group = ps.get(groupattr)
+                    elif hasattr(ps,'getProperty'):
+                        id_for_group = ps.getProperty(groupattr)
+                if id_for_group:
+                    userid = id_for_group #.decode('Latin1').encode('utf-8')
+                    break
+
         result = self.searchGroupsOfUser(self.config['groupdn'],
                                          self.config['attributeid'],
                                          self.config['attributemembers'],
                                          userid,
                                          ALLOWED_SCOPES[self.config['scope']],
                                          objectClass=self.config['objectclasses'])
-
         return tuple(result)
 
     # implements interface IGroupEnumeration (PAS)
@@ -317,13 +321,14 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
                      objectClass=[], titleattr=None):
         """Search for groups in LDAP Directory.
 
-        Take the base DN, the group attribute name, a valid scope from bda.ldap.base,
-        and optional a searchstring, a list of objectClass names and a title
-        attribute defining in which field the group name is stored. If titleattr is
-        given, return a list of tuples containing (id, title) of the groups,
-        otherwise return a list of ids.
+        Take the base DN, the group attribute name, a valid scope from 
+        bda.ldap.base, and optional a searchstring, a list of objectClass names 
+        and a title attribute defining in which field the group name is stored. 
+        If titleattr is given, return a list of tuples containing (id, title) 
+        of the groups, otherwise return a list of ids.
         """
-        view_name = createViewName('GroupsFromLDAPMultiPlugin.searchGroups', self.id)
+        view_name = createViewName('GroupsFromLDAPMultiPlugin.searchGroups', 
+                                   self.id)
         keywords = { 'key': '='.join([str(basedn), str(groupattr), str(scope),
                                       str(search), str(objectClass),
                                       str(titleattr)])
@@ -340,6 +345,7 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
         for oc in objectClass:
             query = '%s(objectClass=%s)' % (query, oc)
         query = '%s)' % query
+        
         try:
             res = self.ldap.search(query, scope)
         except Exception, ex:
@@ -353,7 +359,8 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
             if titleattr is None:
                 ret.append(rec[1].get(groupattr)[0])
             else:
-                ret.append((rec[1].get(groupattr)[0], rec[1].get(titleattr, [None])[0]))
+                ret.append((rec[1].get(groupattr)[0], rec[1].get(titleattr, 
+                                                                 [None])[0]))
 
         self.ZCacheable_set(ret, view_name=view_name, keywords=keywords)
         return ret
@@ -363,13 +370,16 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
                            scope, objectClass=[]):
         """Search for the groups a user is member of.
 
-        Take the base DN, the group attribute name, the user attibute name used in
-        the groups, the user value to search for, a valid scope from bda.ldab.base
-        and optional a list of objectClass names. Return a list of group ids.
+        Take the base DN, the group attribute name, the user attibute name used 
+        in groups, the user value to search for, a valid scope from 
+        bda.ldap.base and optional a list of objectClass names. 
+        Return a list of group ids.
         """
-        view_name = createViewName('GroupsFromLDAPMultiPlugin.searchGroupsOfUser', self.id)
+        view_name = createViewName('GroupsFromLDAPMultiPlugin.searchGroupsOfUser', 
+                                   self.id)
         keywords = {'key': '='.join([str(basedn), str(groupattr), str(userattr),
-                                     str(uservalue), str(scope), str(objectClass)])}
+                                     str(uservalue), str(scope), 
+                                     str(objectClass)])}
 
         cached = self.ZCacheable_get(view_name=view_name,
                                      keywords=keywords,
@@ -401,14 +411,17 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
         return ret
 
     security.declarePrivate('searchMembersOfGroup')
-    def searchMembersOfGroup(self, basedn, groupattr, userattr, groupvalue, scope, objectClass=[]):
+    def searchMembersOfGroup(self, basedn, groupattr, userattr, groupvalue, 
+                             scope, objectClass=[]):
         """Search for the members of a group
 
-        Take the base DN, the group attribute name, the user attibute name used in
-        the groups, the group value to search for, a valid scope from bda.ldab.base
-        and optional a list of objectClass names. Return a list of member ids.
+        Take the base DN, the group attribute name, the user attibute name used 
+        in groups, the group value to search for, a valid scope from 
+        bda.ldap.base and optional a list of objectClass names. 
+        Returns a list of member ids.
         """
-        view_name = createViewName('GroupsFromLDAPMultiPlugin.searchMembersOfGroup', self.id)
+        view_name = createViewName('GroupsFromLDAPMultiPlugin.searchMembersOfGroup', 
+                                   self.id)
         keywords = {'key': '&'.join([basedn, groupattr, userattr, groupvalue,
                                      str(scope), str(objectClass)])}
 
@@ -448,7 +461,8 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
         """
 
         # See if the group can be retrieved from the cache
-        view_name = createViewName('GroupsFromLDAPMultiPlugin.makeGroup', self.id)
+        view_name = createViewName('GroupsFromLDAPMultiPlugin.makeGroup', 
+                                   self.id)
         keywords = { 'group_id' : groupid,
 
                      'title' : title
@@ -460,11 +474,8 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
         if group is not None:
             # cache hit
             group = group.__of__(self)
-            #print "hit cache for: %s" % groupid
             return group
 
-        #print "Missed cache for: %s" % groupid
-
         # verify group and its title from LDAP
         result = self.searchGroups(self.config['groupdn'],
                                    self.config['attributeid'],
@@ -479,7 +490,9 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
         # check if for some reason the wrong group was returned.
         # this must never happen
         if result[0] != groupid:
-            raise ValueError, 'Group with id "%s" expected, but LDAP returned record with id "%s"' % (groupid, id)
+            raise ValueError, 'Group with id '+\
+                  '"%s" expected, but LDAP returned record with id "%s"' % \
+                  (groupid, id)
 
         # create the plone style group object
         group = PloneGroup(groupid)
@@ -511,7 +524,6 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
         # Cache the group if caching is enabled
         base_group = aq_base(group)
         if getattr(base_group, '_p_jar', None) is None:
-            #print "cached group: %s" % groupid
             self.ZCacheable_set(base_group,
                                 view_name=view_name,
                                 keywords=keywords,
@@ -522,8 +534,8 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
     #---------------------------------------------------------------------------
     # helper methods
     def escapeValue(self, query):
-        """ Escapes a query, note that this is documented for AD queries, but not for
-            OpenLDAP etc; But hopefully they work in the same manner.
+        """ Escapes a query, note that this is documented for AD queries, but 
+            not for OpenLDAP etc; But hopefully they work in the same manner.
         """
         config = self.getConfig()
         if not config['escapevalues']:
@@ -585,12 +597,14 @@ class GroupsFromLDAPMultiPlugin(BasePlugin, Cacheable):
             # case adding from setuphandler
             form = REQUEST['form']
         for key in form.keys():
-            if key in self.config.keys():
+            if key in self.config_whitelist:
                 self.config[key] = form.get(key)
                 # special list handling
                 if key == 'objectclasses':
-                    self.config[key] = [c.strip() for c in self.config[key].split(',')]
-                if key == 'scope' and self.config[key] not in ALLOWED_SCOPES.keys():
+                    self.config[key] = [c.strip() for c in \
+                                        self.config[key].split(',')]
+                if key == 'scope' \
+                   and self.config[key] not in ALLOWED_SCOPES.keys():
                     self.config[key] = ALLOWED_SCOPES.keys()[2]
                 if key == 'escapevalues':
                     self.config[key] = self.config[key] == 'yes'
