@@ -1,25 +1,5 @@
-# Copyright (c) 2006 by BlueDynamics Tyrol - Klein und Partner KEG, Austria
-#
+# Copyright (c) 2006-2009 BlueDynamics Alliance, Austria http://bluedynamics.com
 # GNU General Public License (GPL)
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-# 02110-1301, USA.
-#
-
-__author__ = """Robert Niederreiter <robertn@bluedynamics.com>"""
-__docformat__ = 'plaintext'
 
 from Products.PluggableAuthService.interfaces.plugins import IGroupsPlugin
 from Products.PluggableAuthService.interfaces.plugins import \
@@ -54,6 +34,7 @@ def setupPlugin(context):
                                                       'groupid',
                                                       'groupid',
                                                       'uid',
+                                                      None,
                                                       REQUEST=None)
         registry.activatePlugin(IGroupsPlugin, PLUGINID )
         registry.activatePlugin(IGroupEnumerationPlugin, PLUGINID )
